use crate::config::Context;

use std::{io, sync::Arc};

use serde_json;
use tokio::{io::AsyncWriteExt, net::TcpListener, sync::RwLock};

pub async fn apiserver(ctxarc: &Arc<RwLock<Context>>) -> io::Result<()> {
    let ctxarc = ctxarc.clone();
    let addr: String = { ctxarc.read().await.cfg.api.parse().unwrap() };
    let listener = match TcpListener::bind(addr).await {
        Ok(o) => o,
        Err(e) => {
            println!("error: {e}");
            std::process::exit(1);
        }
    };

    tokio::spawn(async move {
        loop {
            match listener.accept().await {
                Ok((mut socket, _addr)) => {
                    let mut buf = vec![0; 1024];

                    match socket.readable().await {
                        Ok(_) => {
                            match socket.try_read(&mut buf) {
                                Ok(_) => {}
                                Err(e) => {
                                    println!("error: {e}");
                                    continue;
                                }
                            };
                        }
                        Err(e) => {
                            println!("error: {e}");
                            continue;
                        }
                    }

                    let msg = match String::from_utf8(buf.to_vec()) {
                        Ok(o) => o.trim_matches(char::from(0)).trim().to_string(),
                        Err(_) => "".to_string(),
                    };

                    let res = format_result(&ctxarc, msg.as_str()).await;

                    match socket.write_all(res.as_bytes()).await {
                        Ok(_) => {}
                        Err(e) => {
                            println!("error: {e}");
                        }
                    }
                }
                Err(err) => {
                    println!("error: {err}");
                }
            }
        }
    });
    Ok(())
}

async fn format_result(ctxarc: &Arc<RwLock<Context>>, mode: &str) -> String {
    let data;
    let ctx = ctxarc.read().await;
    match mode {
        "cfg" => data = serde_json::to_string(&ctx.cfg).unwrap(),
        "blocklist" => data = serde_json::to_string(&ctx.blocklist).unwrap(),
        _ => data = serde_json::to_string(&ctx.blocklist).unwrap(),
    };
    data
}
